@@ -12,7 +12,20 @@ Rex - Remote Execution
 
 =head1 DESCRIPTION
 
-(R)?ex is a small script to ease the execution of remote commands. You can write small tasks in a file named I<Rexfile>.
+Rex is a command line tool which executes commands on remote servers.  Define
+tasks in Perl and execute them on remote servers or groups of servers.
+
+Rex can be used to:
+
+=over 4
+
+=item * Deploy web applications to servers sequentially or in parallel.
+
+=item * Automate common tasks.
+
+=item * Provision servers using Rex's builtin tools.
+
+=back
 
 You can find examples and howtos on L<http://rexify.org/>
 
@@ -32,18 +45,23 @@ You can find examples and howtos on L<http://rexify.org/>
 
 =head1 SYNOPSIS
 
- use strict;
- use warnings;
+    # In a Rexfile:
+    use Rex -feature => [qw/1.3/];
+   
+    user "root";
+    password "ch4ngem3";
+   
+    desc "Show Unix version";
+    task "uname", sub {
+       say run "uname -a";
+    };
 
- user "root";
- password "ch4ngem3";
+    1;
+   
+    # On the command line:
+    bash# rex -H server[01..10] uname
 
- desc "Show Unix version";
- task "uname", sub {
-    say run "uname -a";
- };
-
- bash# rex -H "server[01..10]" uname
+See L<rex|https://metacpan.org/pod/distribution/Rex/bin/rex> for more information about how to use rex on the command line.
 
 See L<Rex::Commands> for a list of all commands you can use.
 
@@ -56,11 +74,11 @@ package Rex;
 use strict;
 use warnings;
 
-our $VERSION = '1.3.3'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 # development version if this variable is not set
 if ( !$Rex::VERSION ) {
-  $Rex::VERSION = "9999.99.99";
+  $Rex::VERSION = "9999.99.99_99";
 }
 
 BEGIN {
@@ -75,18 +93,100 @@ BEGIN {
   use Rex::Notify;
   use Rex::Require;
   use File::Basename;
+  use File::Spec;
   eval { Net::SSH2->require; };
 }
 
 our ( @EXPORT, @CONNECTION_STACK, $GLOBAL_SUDO, $MODULE_PATHS,
-  $WITH_EXIT_STATUS );
+  $WITH_EXIT_STATUS, @FEATURE_FLAGS );
 
 $WITH_EXIT_STATUS = 1; # since 0.50 activated by default
+@FEATURE_FLAGS    = ();
 
 my $cur_dir;
 
 BEGIN {
 
+  sub generate_inc {
+    my @additional = @_;
+
+    my @rex_inc = ();
+
+# this must be the first, special handling for rex modules which uses Module.pm and not
+# __module__.pm as their initial file. (rex pre 0.40 or something)
+    push @rex_inc, sub {
+      my $mod_to_load = $_[1];
+      return search_module_path( $mod_to_load, 1 );
+    };
+
+# this adds the current directory to the lib search path.
+# this must come before all other paths, because custom libraries can be project dependant
+# see: #1108
+    push @rex_inc, add_cwd_to_inc();
+    push @rex_inc, add_libstruct_to_inc($_) for @additional;
+
+    # we have to add the Rexfile's path to @INC FIX: #1170
+    push @rex_inc, @additional;
+
+# add home directory/.rex/recipes to the search path, so that recipes can be managed
+# at a central location.
+    my $home_dir = _home_dir();
+    if ( -d "$home_dir/.rex/recipes" ) {
+      push( @INC, "$home_dir/.rex/recipes" );
+    }
+
+    # add the default search locations
+    push @rex_inc, @INC;
+
+    # this must be the last entry, special handling to load rex modules.
+    push(
+      @rex_inc,
+      sub {
+        my $mod_to_load = $_[1];
+        return search_module_path( $mod_to_load, 0 );
+      }
+    );
+
+    return @rex_inc;
+  }
+
+  sub add_libstruct_to_inc {
+    my ($path) = @_;
+    my @ret = ();
+
+    if ( -d File::Spec->catdir( $path, "lib" ) ) {
+      push( @ret, File::Spec->catdir( $path, "lib" ) );
+      push( @ret, File::Spec->catdir( $path, "lib", "perl", "lib", "perl5" ) );
+      if ( $^O eq "linux" ) {
+        push(
+          @ret,
+          File::Spec->catdir(
+            $path, "lib", "perl", "lib", "perl5", "x86_64-linux"
+          )
+        );
+      }
+      if ( $^O =~ m/^MSWin/ ) {
+        my ($special_win_path) = grep { m/\/MSWin32\-/ } @INC;
+        if ( defined $special_win_path ) {
+          my $mswin32_path = basename $special_win_path;
+          push(
+            @ret,
+            File::Spec->catdir(
+              $path, "lib", "perl", "lib", "perl5", $mswin32_path
+            )
+          );
+        }
+      }
+    }
+
+    return @ret;
+  }
+
+  sub add_cwd_to_inc {
+    my $path = getcwd;
+    return add_libstruct_to_inc($path);
+  }
+
   sub _home_dir {
     if ( $^O =~ m/^MSWin/ ) {
       return $ENV{'USERPROFILE'};
@@ -95,40 +195,8 @@ BEGIN {
     return $ENV{'HOME'} || "";
   }
 
-  $cur_dir = getcwd;
-
-  unshift(
-    @INC,
-    sub {
-      my $mod_to_load = $_[1];
-      return search_module_path( $mod_to_load, 1 );
-    }
-  );
-
-  if ( -d "$cur_dir/lib" ) {
-    push( @INC, "$cur_dir/lib" );
-    push( @INC, "$cur_dir/lib/perl/lib/perl5" );
-    if ( $^O =~ m/^MSWin/ ) {
-      my ($special_win_path) = grep { m/\/MSWin32\-/ } @INC;
-      if ( defined $special_win_path ) {
-        my $mswin32_path = basename $special_win_path;
-        push( @INC, "$cur_dir/lib/perl/lib/perl5/$mswin32_path" );
-      }
-    }
-  }
-
-  my $home_dir = _home_dir();
-  if ( -d "$home_dir/.rex/recipes" ) {
-    push( @INC, "$home_dir/.rex/recipes" );
-  }
-
-  push(
-    @INC,
-    sub {
-      my $mod_to_load = $_[1];
-      return search_module_path( $mod_to_load, 0 );
-    }
-  );
+  my @new_inc = generate_inc();
+  @INC = @new_inc;
 
 }
 
@@ -157,7 +225,16 @@ sub search_module_path {
   }
 
   for my $file (@search_in) {
-    if ( -f $file ) {
+    my $o = -f $file;
+    my $fh_t;
+    if ( $^O =~ m/^MSWin/i && !$o ) {
+
+      # this is a windows workaround for if(-f ) on symlinks
+      $o = open( my $fh_t, "<", $file );
+    }
+
+    if ($o) {
+      close $fh_t if $fh_t;
       my ($path) = ( $file =~ m/^(.*)\/.+?$/ );
       if ( $path !~ m/\// ) {
         $path = $cur_dir . "/$path";
@@ -311,13 +388,19 @@ Returns 1 if the current operation is executed within sudo.
 =cut
 
 sub is_sudo {
-  if ($GLOBAL_SUDO) { return 1; }
 
   if ( exists $CONNECTION_STACK[-1]->{server}->{auth}->{sudo}
     && $CONNECTION_STACK[-1]->{server}->{auth}->{sudo} == 1 )
   {
     return 1;
   }
+  elsif ( exists $CONNECTION_STACK[-1]->{server}->{auth}->{sudo}
+    && $CONNECTION_STACK[-1]->{server}->{auth}->{sudo} == 0 )
+  {
+    return 0;
+  }
+
+  if ($GLOBAL_SUDO) { return 1; }
 
   if ( $CONNECTION_STACK[-1] ) {
     return $CONNECTION_STACK[-1]->{conn}->get_current_use_sudo;
@@ -462,9 +545,44 @@ sub deprecated {
 
 }
 
+sub has_feature_version {
+  my ($version) = @_;
+
+  my @version_flags = grep { m/^\d+\./ } @FEATURE_FLAGS;
+  for my $v (@version_flags) {
+    if ( $version <= $v ) {
+      return 1;
+    }
+  }
+
+  return 0;
+}
+
+sub has_feature_version_lower {
+  my ($version) = @_;
+
+  my @version_flags = grep { m/^\d+\./ } @FEATURE_FLAGS;
+  for my $v (@version_flags) {
+    if ( $version > $v ) {
+      return 1;
+    }
+  }
+
+  return 0;
+}
+
 sub import {
   my ( $class, $what, $addition1 ) = @_;
 
+  srand();
+
+  if ( $addition1 && ref $addition1 eq "ARRAY" ) {
+    push @FEATURE_FLAGS, $what, @{$addition1};
+  }
+  elsif ($addition1) {
+    push @FEATURE_FLAGS, $what, $addition1;
+  }
+
   $what ||= "";
 
   my ( $register_to, $file, $line ) = caller;
@@ -472,6 +590,14 @@ sub import {
   # use Net::OpenSSH if present (default without feature flag)
   Rex::Config->set_use_net_openssh_if_present(1);
 
+  if ( $what eq "-minimal" ) {
+    require Rex::Commands;
+    Rex::Commands->import( register_in => $register_to );
+
+    require Rex::Helper::Rexfile::ParamLookup;
+    Rex::Helper::Rexfile::ParamLookup->import( register_in => $register_to );
+  }
+
   if ( $what eq "-base" || $what eq "base" || $what eq "-feature" ) {
     require Rex::Commands;
     Rex::Commands->import( register_in => $register_to );
@@ -527,6 +653,11 @@ sub import {
     require Rex::Commands::User;
     Rex::Commands::User->import( register_in => $register_to );
 
+    require Rex::Helper::Rexfile::ParamLookup;
+    Rex::Helper::Rexfile::ParamLookup->import( register_in => $register_to );
+
+    require Rex::Resource::firewall;
+    Rex::Resource::firewall->import( register_in => $register_to );
   }
 
   if ( $what eq "-feature" || $what eq "feature" ) {
@@ -540,23 +671,34 @@ sub import {
 
       if ( $add =~ m/^(\d+\.\d+)$/ ) {
         my $vers = $1;
-        my $_ver = $Rex::VERSION;
-        $_ver =~ s/_\d+$//; # remove rc info
+        my ( $major, $minor, $patch, $dev_release ) =
+          $Rex::VERSION =~ m/^(\d+)\.(\d+)\.(\d+)_?(\d+)?$/;
 
-        my ( $major, $minor, $patch ) = split( /\./, $_ver );
         my ( $c_major, $c_minor ) = split( /\./, $vers );
 
-        if ( ( $c_major > $major )
+        if ( defined $dev_release && $c_major == $major && $c_minor > $minor ) {
+          Rex::Logger::info(
+            "This is development release $Rex::VERSION of Rex. Enabling experimental feature flag for $vers.",
+            "warn"
+          );
+        }
+        elsif ( ( $c_major > $major )
           || ( $c_major >= $major && $c_minor > $minor ) )
         {
           Rex::Logger::info(
             "This Rexfile tries to enable features that are not supported with your version. Please update.",
-            "warn"
+            "error"
           );
           exit 1;
         }
       }
 
+      if ( $add =~ m/^\d+\.\d+$/ && $add >= 1.4 ) {
+        Rex::Logger::debug("Enabling task_chaining_cmdline_args feature");
+        Rex::Config->set_task_chaining_cmdline_args(1);
+        $found_feature = 1;
+      }
+
       if ( $add =~ m/^\d+\.\d+$/ && $add >= 1.3 ) {
         Rex::Logger::debug("Activating new template engine.");
         Rex::Config->set_use_template_ng(1);
@@ -786,6 +928,18 @@ sub import {
         $found_feature = 1;
       }
 
+      if ( $add eq "no_task_chaining_cmdline_args" ) {
+        Rex::Logger::debug("Disabling task_chaining_cmdline_args feature");
+        Rex::Config->set_task_chaining_cmdline_args(0);
+        $found_feature = 1;
+      }
+
+      if ( $add eq "task_chaining_cmdline_args" ) {
+        Rex::Logger::debug("Enabling task_chaining_cmdline_args feature");
+        Rex::Config->set_task_chaining_cmdline_args(1);
+        $found_feature = 1;
+      }
+
       if ( $found_feature == 0 ) {
         Rex::Logger::info(
           "You tried to load a feature ($add) that doesn't exists in your Rex version. Please update.",
