@@ -62,7 +62,7 @@ use strict;
 use warnings;
 use Fcntl;
 
-our $VERSION = '1.4.1'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 require Rex::Exporter;
 use Data::Dumper;
@@ -70,11 +70,9 @@ use Rex::Config;
 use Rex::FS::File;
 use Rex::Commands::Upload;
 use Rex::Commands::MD5;
-use Rex::Commands::Run;
 use Rex::File::Parser::Data;
 use Rex::Helper::System;
 use Rex::Helper::Path;
-use Rex::Helper::Run;
 use Rex::Hook;
 use Carp;
 
@@ -1040,6 +1038,12 @@ sub _append_or_update {
     }
   };
 
+  unless ( defined $new_line ) {
+    my ( undef, undef, undef, $subroutine ) = caller(1);
+    $subroutine =~ s/^.*:://;
+    die "Undefined new line while trying to run $subroutine on $file";
+  }
+
   my $fs = Rex::Interface::Fs->create;
 
   my ( $old_md5, $ret );
