@@ -30,7 +30,7 @@ package Rex::Commands::Tail;
 use strict;
 use warnings;
 
-our $VERSION = '1.3.3'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 require Rex::Exporter;
 use Data::Dumper;
@@ -79,26 +79,11 @@ sub tail {
 
   Rex::Logger::debug("Tailing: $file");
 
-  my $int =
-    Rex::Commands::get("rex_internals") || { read_buffer_size => 1024, };
-
-  my $old_buf_size = $int->{read_buffer_size} || 1024;
-
-  Rex::Commands::set(
-    "rex_internals",
-    {
-      read_buffer_size => 1,
-    }
-  );
-
   run "tail -f $file", continuous_read => sub {
     $callback->(@_);
     },
     ;
 
-  $int->{read_buffer_size} = $old_buf_size;
-  Rex::Commands::set( "rex_internals", $int );
-
 }
 
 1;
