@@ -23,7 +23,7 @@ package Rex::Config;
 use strict;
 use warnings;
 
-our $VERSION = '1.3.3'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 use Rex::Helper::File::Spec;
 use Rex::Logger;
@@ -60,7 +60,7 @@ our (
   $register_cmdb_template,   $check_service_exists,
   $set_no_append,            $use_net_openssh_if_present,
   $use_template_ng,          $use_rex_kvm_agent,
-  $autodie,
+  $autodie,                  $task_chaining_cmdline_args,
 
 );
 
@@ -162,6 +162,15 @@ sub get_disable_taskname_warning {
   return $disable_taskname_warning;
 }
 
+sub set_task_chaining_cmdline_args {
+  my $class = shift;
+  $task_chaining_cmdline_args = shift;
+}
+
+sub get_task_chaining_cmdline_args {
+  return $task_chaining_cmdline_args;
+}
+
 sub set_verbose_run {
   my $class = shift;
   $verbose_run = shift;
@@ -441,7 +450,12 @@ sub get_user {
     return $user;
   }
 
-  return getlogin || getpwuid($<) || "Kilroy";
+  if ( $^O =~ m/^MSWin/ ) {
+    return getlogin;
+  }
+  else {
+    return getpwuid($<);
+  }
 }
 
 sub get_password {
@@ -665,7 +679,12 @@ sub get_ssh_config_hostname {
     && exists $SSH_CONFIG_FOR{ $param->{server} }
     && exists $SSH_CONFIG_FOR{ $param->{server} }->{hostname} )
   {
-    return $SSH_CONFIG_FOR{ $param->{server} }->{hostname};
+    if ( $SSH_CONFIG_FOR{ $param->{server} }->{hostname} =~ m/^\%h\.(.*)/ ) {
+      return $param->{server} . "." . $1;
+    }
+    else {
+      return $SSH_CONFIG_FOR{ $param->{server} }->{hostname};
+    }
   }
 
   return 0;
@@ -776,7 +795,20 @@ sub set_template_function {
 
 sub get_template_function {
   if ( ref($template_function) eq "CODE" ) {
-    return $template_function;
+    return sub {
+      my ( $content, $template_vars ) = @_;
+      $template_vars = {
+        Rex::Commands::task()->get_opts,
+        (
+          Rex::Resource->is_inside_resource
+          ? %{ Rex::Resource->get_current_resource()->get_all_parameters }
+          : ()
+        ),
+        %{ $template_vars || {} }
+        }
+        if ( Rex::Commands::task() );
+      return $template_function->( $content, $template_vars );
+    };
   }
 
   if ( Rex::Template::NG->is_loadable && get_use_template_ng() ) {
@@ -784,6 +816,16 @@ sub get_template_function {
     # new template engine
     return sub {
       my ( $content, $template_vars ) = @_;
+      $template_vars = {
+        Rex::Commands::task()->get_opts,
+        (
+          Rex::Resource->is_inside_resource
+          ? %{ Rex::Resource->get_current_resource()->get_all_parameters }
+          : ()
+        ),
+        %{ $template_vars || {} }
+        }
+        if ( Rex::Commands::task() );
       Rex::Template::NG->require;
       my $t = Rex::Template::NG->new;
       return $t->parse( $content, %{$template_vars} );
@@ -792,6 +834,16 @@ sub get_template_function {
 
   return sub {
     my ( $content, $template_vars ) = @_;
+    $template_vars = {
+      Rex::Commands::task()->get_opts,
+      (
+        Rex::Resource->is_inside_resource
+        ? %{ Rex::Resource->get_current_resource()->get_all_parameters }
+        : ()
+      ),
+      %{ $template_vars || {} }
+      }
+      if ( Rex::Commands::task() );
     use Rex::Template;
     my $template = Rex::Template->new;
     return $template->parse( $content, $template_vars );
@@ -865,6 +917,7 @@ sub unset {
 
 sub get {
   my ( $class, $var ) = @_;
+  $var or return;
   if ( exists $set_param->{$var} ) {
     return $set_param->{$var};
   }
