@@ -9,10 +9,9 @@ package Rex::Hardware::Network::Solaris;
 use strict;
 use warnings;
 
-our $VERSION = '1.3.3'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 use Rex::Logger;
-use Rex::Commands::Run;
 use Rex::Helper::Run;
 use Rex::Helper::Array;
 
@@ -55,7 +54,7 @@ sub route {
 
   my @ret = ();
 
-  my @route = i_run "netstat -nr";
+  my @route = i_run "netstat -nr", fail_ok => 1;
   if ( $? != 0 ) {
     die("Error running netstat");
   }
@@ -100,13 +99,13 @@ sub default_gateway {
 
   if ($new_default_gw) {
     if ( default_gateway() ) {
-      i_run "route delete default " . default_gateway();
+      i_run "route delete default " . default_gateway(), fail_ok => 1;
       if ( $? != 0 ) {
         die("Error running route del default");
       }
     }
 
-    i_run "route add default $new_default_gw";
+    i_run "route add default $new_default_gw", fail_ok => 1;
     if ( $? != 0 ) {
       die("Error route add default");
     }
@@ -127,7 +126,7 @@ sub default_gateway {
 sub netstat {
 
   my @ret;
-  my @netstat = i_run "netstat -na -f inet -f inet6";
+  my @netstat = i_run "netstat -na -f inet -f inet6", fail_ok => 1;
   if ( $? != 0 ) {
     die("Error running netstat");
   }
@@ -295,7 +294,7 @@ sub netstat {
 
   }
 
-  @netstat = i_run "netstat -na -f unix";
+  @netstat = i_run "netstat -na -f unix", fail_ok => 1;
   shift @netstat;
   shift @netstat;
   shift @netstat;
