@@ -9,7 +9,7 @@ package Rex::Helper::File::Stat::Unix;
 use strict;
 use warnings;
 
-our $VERSION = '1.3.3'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 use Fcntl;
 
@@ -50,4 +50,9 @@ sub S_ISSOCK {
   Fcntl::S_ISSOCK(@_);
 }
 
+sub S_IMODE {
+  shift;
+  Fcntl::S_IMODE(@_);
+}
+
 1;
