@@ -11,7 +11,7 @@ use warnings;
 use Rex::Helper::Run;
 use Rex::Interface::Exec;
 
-our $VERSION = '1.3.3'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 sub new {
   my $that  = shift;
@@ -72,7 +72,7 @@ sub update {
       $option->{version};
   }
 
-  my $f = i_run $cmd;
+  my $f = i_run $cmd, fail_ok => 1;
 
   unless ( $? == 0 ) {
     Rex::Logger::info( "Error installing $pkg.", "warn" );
@@ -86,20 +86,43 @@ sub update {
 }
 
 sub update_system {
-  my ($self) = @_;
+  my ( $self, %option ) = @_;
+
+  # default is to update packages
+  $option{update_packages} = 1 if ( !exists $option{update_packages} );
 
   if ( !exists $self->{commands}->{update_system} ) {
     Rex::Logger::debug("Not supported under this OS");
     return;
   }
 
-  my $cmd = $self->{commands}->{update_system};
-  my $f   = i_run $cmd;
+  if ( $option{update_metadata} ) {
+    $self->update_pkg_db(%option);
+  }
 
-  unless ( $? == 0 ) {
-    Rex::Logger::info( "Error updating system.", "warn" );
-    Rex::Logger::debug($f);
-    die("Error updating system");
+  if ( $option{update_packages} ) {
+    my $cmd = $self->{commands}->{update_system};
+    my $f = i_run $cmd, fail_ok => 1;
+
+    unless ( $? == 0 ) {
+      Rex::Logger::debug($f);
+      die("Error updating system");
+    }
+  }
+
+  if ( $option{dist_upgrade} ) {
+    if ( !exists $self->{commands}->{dist_update_system} ) {
+      Rex::Logger::debug("dist upgrades not supported under this OS");
+    }
+    else {
+      my $cmd = $self->{commands}->{dist_update_system};
+      my $f = i_run $cmd, fail_ok => 1;
+
+      unless ( $? == 0 ) {
+        Rex::Logger::debug($f);
+        die("Error dist-updating system");
+      }
+    }
   }
 
   Rex::Logger::debug("System successfully updated.");
@@ -113,7 +136,7 @@ sub remove {
   Rex::Logger::debug("Removing $pkg");
   my $cmd = sprintf $self->{commands}->{remove}, $pkg;
 
-  my $f = i_run $cmd;
+  my $f = i_run $cmd, fail_ok => 1;
 
   unless ( $? == 0 ) {
     Rex::Logger::info( "Error removing $pkg.", "warn" );
@@ -132,7 +155,7 @@ sub purge {
   Rex::Logger::debug("Purging $pkg");
   my $cmd = sprintf $self->{commands}->{purge}, $pkg;
 
-  my $f = i_run $cmd;
+  my $f = i_run $cmd, fail_ok => 1;
 
   unless ( $? == 0 ) {
     Rex::Logger::info( "Error purging $pkg.", "warn" );
@@ -146,7 +169,7 @@ sub purge {
 }
 
 sub update_pkg_db {
-  my ($self) = @_;
+  my ( $self, %option ) = @_;
 
   if ( !exists $self->{commands}->{update_package_db} ) {
     Rex::Logger::debug("Not supported under this OS");
@@ -154,7 +177,7 @@ sub update_pkg_db {
   }
 
   my $cmd = $self->{commands}->{update_package_db};
-  i_run $cmd;
+  i_run $cmd, fail_ok => 1;
   if ( $? != 0 ) {
     die("Error updating package database");
   }
