@@ -9,11 +9,9 @@ package Rex::Pkg::Ubuntu;
 use strict;
 use warnings;
 
-our $VERSION = '1.3.3'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 use Rex::Pkg::Debian;
-use Rex::Commands::Run;
-use Rex::Commands::File;
 
 use base qw(Rex::Pkg::Debian);
 
