@@ -9,13 +9,15 @@ package Rex::Resource;
 use strict;
 use warnings;
 
-our $VERSION = '1.3.3'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 use Rex::Constants;
 
-our $INSIDE_RES = 0;
 our @CURRENT_RES;
 
+sub is_inside_resource { ref $CURRENT_RES[-1] ? 1 : 0 }
+sub get_current_resource { $CURRENT_RES[-1] }
+
 sub new {
   my $that  = shift;
   my $proto = ref($that) || $that;
@@ -45,23 +47,16 @@ sub call {
     return;
   }
 
-  $INSIDE_RES = 1;
   push @CURRENT_RES, $self;
 
+  $self->set_all_parameters(%params);
+
   $self->{res_name} = $name;
   $self->{res_ensure} = $params{ensure} ||= present;
 
   Rex::get_current_connection()->{reporter}
     ->report_resource_start( type => $self->display_name, name => $name );
 
-  # make parameters automatically availabel in templates
-  # store old values so that we can recover them after resource finish
-  my %old_values;
-  for my $key ( keys %params ) {
-    $old_values{$key} = Rex::Config->get($key);
-    Rex::Config->set( $key => $params{$key} );
-  }
-
   my $failed = 0;
   eval {
     $self->{cb}->( \%params );
@@ -94,18 +89,7 @@ sub call {
   Rex::get_current_connection()->{reporter}
     ->report_resource_end( type => $self->display_name, name => $name );
 
-  $INSIDE_RES = 0;
   pop @CURRENT_RES;
-
-  # recover old values
-  for my $key ( keys %old_values ) {
-    if ( defined $old_values{$key} ) {
-      Rex::Config->set( $key => $old_values{$key} );
-    }
-    else {
-      Rex::Config->set( $key => undef );
-    }
-  }
 }
 
 sub was_updated {
@@ -162,4 +146,19 @@ sub message {
   }
 }
 
+sub set_parameter {
+  my ( $self, $key, $value ) = @_;
+  $self->{__res_parameters__}->{$key} = $value;
+}
+
+sub set_all_parameters {
+  my ( $self, %params ) = @_;
+  $self->{__res_parameters__} = \%params;
+}
+
+sub get_all_parameters {
+  my ($self) = @_;
+  return $self->{__res_parameters__};
+}
+
 1;
