@@ -11,11 +11,11 @@ package Rex::Sudo::File;
 use strict;
 use warnings;
 
-our $VERSION = '1.3.3'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 use Rex;
 use Rex::Commands;
-use Rex::Commands::Run;
+use Rex::Helper::Run;
 use Rex::Commands::Fs;
 use Rex::Helper::Path;
 use IO::File;
@@ -103,7 +103,7 @@ sub close {
 
   # use cat to not overwrite attributes/owner/group
   if ( $self->{mode} eq ">" || $self->{mode} eq ">>" ) {
-    run "cat " . $self->{rndfile} . " >" . $self->{file};
+    i_run "cat " . $self->{rndfile} . " >" . $self->{file};
     rm( $self->{rndfile} );
   }
 }
