@@ -9,7 +9,7 @@ package Rex::Virtualization::LibVirt::delete;
 use strict;
 use warnings;
 
-our $VERSION = '1.4.1'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 use Rex::Logger;
 use Rex::Helper::Run;
@@ -31,7 +31,7 @@ sub execute {
     die("VM $dom not found.");
   }
 
-  i_run "virsh -c $uri undefine '$dom'";
+  i_run "virsh -c $uri undefine '$dom'", fail_ok => 1;
   if ( $? != 0 ) {
     die("Error destroying vm $dom");
   }
