@@ -9,7 +9,7 @@ package Rex::Virtualization::LibVirt::guestinfo;
 use strict;
 use warnings;
 
-our $VERSION = '1.4.1'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 use Data::Dumper;
 use Rex::Logger;
@@ -20,7 +20,7 @@ use Rex::Commands::Gather;
 use Rex::Virtualization::LibVirt::info;
 use Rex::Interface::File;
 use Rex::Interface::Exec;
-use JSON::XS;
+use JSON::MaybeXS;
 require Rex::Commands::Run;
 
 sub execute {
@@ -144,7 +144,7 @@ sub execute {
         map {
         my @x = ( $_ =~ m/\(([^\)]+)\) at ([^\s]+)\s/ );
         ( $x[1], $x[0] )
-        } i_run "$command -an";
+        } i_run "$command -an", fail_ok => 1;
 
       for my $if ( keys %{$ifs} ) {
         if ( exists $arp{ $ifs->{$if}->{mac} } && $arp{ $ifs->{$if}->{mac} } ) {
