@@ -9,7 +9,7 @@ package Rex::Virtualization::LibVirt::vncdisplay;
 use strict;
 use warnings;
 
-our $VERSION = '1.4.1'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 use Rex::Logger;
 use Rex::Helper::Run;
@@ -32,7 +32,7 @@ sub execute {
 
   my $xml;
 
-  my @vncdisplay = i_run "virsh -c $uri vncdisplay '$vmname'";
+  my @vncdisplay = i_run "virsh -c $uri vncdisplay '$vmname'", fail_ok => 1;
 
   if ( $? != 0 ) {
     die("Error running virsh vncdisplay '$vmname'");
