@@ -9,7 +9,7 @@ package Rex::Virtualization::VBox::delete;
 use strict;
 use warnings;
 
-our $VERSION = '1.3.3'; # VERSION
+our $VERSION = '1.6.0'; # VERSION
 
 use Rex::Logger;
 use Rex::Helper::Run;
@@ -28,7 +28,7 @@ sub execute {
     die("VM $dom not found.");
   }
 
-  i_run "VBoxManage unregistervm \"$dom\" --delete";
+  i_run "VBoxManage unregistervm \"$dom\" --delete", fail_ok => 1;
   if ( $? != 0 ) {
     die("Error destroying vm $dom");
   }
